from hoiio.service import Service

from hoiio.service import api_endpoint



class Sms(Service):
    """ 
    Provide SMS services such as sending and querying for SMS.

    Usage: Hoiio.sms.send(...), Hoiio.sms.history(...), etc.
    """
    
    def __init__(self):
        pass

    def send(self, msg, dest, **kwargs):
        """
        Send a SMS to a phone

        :param msg: The SMS message
        :param dest: The phone number to send SMS to
        
        :param sender_name: The sender name that will be displayed to :data:`dest`
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to SMS to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'Sending [%s] to %s' % (msg, dest)
        kwargs['dest'] = dest
        kwargs['msg'] = msg
        return self.make_request(api_endpoint('sms', 'send'), **kwargs)
        

    def bulk_send(self, msg, *dests, **kwargs):
        """
        Send SMS to multiple phone numbers (up to 1,000).

        :param dests: List of phone numbers to send SMS to
        :param msg: The SMS message
        
        :param sender_name: The sender name that will be displayed to :data:`dest`
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to SMS to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'Sending [%s] to %s' % (msg, dests)
        kwargs['dest'] = ','.join(dests)
        kwargs['msg'] = msg
        return self.make_request(api_endpoint('sms', 'bulk_send'), **kwargs)
        

    def history(self, **kwargs):
        """
        Retrieve the history of SMS. There is pagination, and each page returns up to 100 entries.

        :param string from: SMS sent/received after this date. In "YYYY-MM-DD HH:MM:SS" (GMT+8) format.
        :param string to: SMS sent/received made before this date. In "YYYY-MM-DD HH:MM:SS" (GMT+8) format.
        :param int page: The page number. Count starts from 1.
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'SMS history'
        return self.make_request(api_endpoint('sms', 'get_history'), **kwargs)


    def rate(self, msg, dest, **kwargs):
        """
        Retrieve the cost of sending an SMS

        :param msg: The SMS message
        :param dest: The phone number to send SMS to
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'SMS rate from %s [%s]' % (dest, msg)
        kwargs['dest'] = dest
        kwargs['msg'] = msg
        return self.make_request(api_endpoint('sms', 'get_rate'), **kwargs)

    def rate_in(self, dest, **kwargs):
        """
        Retrieve the cost of sending an SMS

        :param dest: The phone number to receive the SMS at
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'SMS rate receiving at %s' % (dest)
        kwargs['incoming'] = dest
        return self.make_request(api_endpoint('sms', 'get_rate'), **kwargs)


    def status(self, txn_ref, **kwargs):
        """
        Retrieve the status and various information about an SMS

        :param txn_ref: The transaction reference for the SMS
        
        :returns: Return :class:`hoiio.service.Response`
        """     
        print 'Status of %s' % (txn_ref)
        kwargs['txn_ref'] = txn_ref
        return self.make_request(api_endpoint('sms', 'query_status'), **kwargs)
       



from hoiio.service import Service
from hoiio.service import api_endpoint

import base64

class Fax(Service):
    """ 
    Provide Fax services such as sending and querying for Fax.

    Usage: Hoiio.fax.send(...), Hoiio.fax.history(...), etc.
    """
    
    def __init__(self):
        pass

    def send(self, dest, filename, **kwargs):
        """
        Send a fax to a fax number

        :param dest: The fax number to send fax to
        :param file: The path to the pdf file to fax
        
        :param caller_id: The caller ID when connected to the fax number
        :param filename: The filename to store on Hoiio. If omitted, the filename of the file will be used.
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'Faxing [%s] to %s' % (filename, dest)
        f = open(filename, "r")
        data = f.read()
        str_data = base64.b64encode(data)

        kwargs['dest'] = dest
        kwargs['file'] = str_data
        return self.make_request(api_endpoint('fax', 'send'), **kwargs)
        

    def history(self, **kwargs):
        """
        Retrieve the history of Fax. There is pagination, and each page returns up to 100 entries.

        :param string from: Fax sent/received after this date. In "YYYY-MM-DD HH:MM:SS" (GMT+8) format.
        :param string to: Fax sent/received before this date. In "YYYY-MM-DD HH:MM:SS" (GMT+8) format.
        :param int page: The page number. Count starts from 1.
        :param type: The type of fax. Default to all.
        :type type: incoming, outgoing or all

        :returns: Return :class:`hoiio.service.Response`
        """
        print 'Fax history'
        return self.make_request(api_endpoint('fax', 'get_history'), **kwargs)


    def rate(self, dest, **kwargs):
        """
        Retrieve the cost of sending a fax

        :param dest: The fax number to send to
        
        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['dest'] = dest
        return self.make_request(api_endpoint('fax', 'get_rate'), **kwargs)


    def rate_in(self, dest, **kwargs):
        """
        Retrieve the cost of receiving a fax

        :param dest: The fax number to receiving at
        
        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['incoming'] = dest
        return self.make_request(api_endpoint('fax', 'get_rate'), **kwargs)


    def status(self, txn_ref, **kwargs):
        """
        Retrieve the status and various information about a fax

        :param txn_ref: The transaction reference for the fax
        
        :returns: Return :class:`hoiio.service.Response`
        """     
        print 'Status of %s' % (txn_ref)
        kwargs['txn_ref'] = txn_ref
        return self.make_request(api_endpoint('fax', 'query_status'), **kwargs)




from hoiio.service import Service

from hoiio.service import api_endpoint


class Voice(Service):
    """ 
    Provide voice services such as making a call back, conference call, and query for calls. 

    Usage: Hoiio.voice.call(...), Hoiio.voice.conference(...), etc.
    """
    
    def __init__(self):
        pass

    def call(self, dest1, dest2, **kwargs):
        """
        Call 2 phone numbers and connect them up

        :param dest1: The first phone number to call. If None, Hoiio will call the account's registered mobile number. 
        :param dest2: The second phont number to call
        :param caller_id: The caller ID to show to :data:`dest2`
        :param max_duration: Maximum duration (in seconds). Call will be hangup after :data:`max_duration`
        
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'Calling %s to %s' % (dest1, dest2)
        kwargs['dest1'] = dest1
        kwargs['dest2'] = dest2
        return self.make_request(api_endpoint('voice', 'call'), **kwargs)
        

    def conference(self, *dests, **kwargs):
        """
        Call multiple phone numbers and connect them up in a conference call

        :param dests: List of phone numbers to call
        
        :param string room: The conference room to transfer the callers to
        :param caller_id: The caller ID to show to :data:`dest2`
        
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'Conference call to %s' % (dests,)
        kwargs['dest'] = ','.join(dests)
        return self.make_request(api_endpoint('voice', 'conference'), **kwargs)


    def hangup(self, txn_ref, **kwargs):
        """
        Hangup a call. For conference, you need to hangup for each of the participants.

        :param txn_ref: The transaction reference for the call to hangup

        :returns: Return :class:`hoiio.service.Response`
        """
        print 'Hangup %s' % (txn_ref)
        kwargs['txn_ref'] = txn_ref
        return self.make_request(api_endpoint('voice', 'hangup'), **kwargs)


    def history(self, **kwargs):
        """
        Retrieve the history of calls. There is pagination, and each page returns up to 100 entries.

        :param string from: Calls made after this date. In "YYYY-MM-DD HH:MM:SS" (GMT+8) format.
        :param string to: Calls made before this date. In "YYYY-MM-DD HH:MM:SS" (GMT+8) format.
        :param int page: The page number. Count starts from 1.
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'Call history'
        return self.make_request(api_endpoint('voice', 'get_history'), **kwargs)


    def rate(self, dest1, dest2, **kwargs):
        """
        Retrieve the cost of making a call

        :param dest1: The first phone number to call. 
        :param dest2: The second phont number to call
        
        :returns: Return :class:`hoiio.service.Response`
        """
        print 'Calling %s to %s' % (dest1, dest2)
        kwargs['dest1'] = dest1
        kwargs['dest2'] = dest2
        return self.make_request(api_endpoint('voice', 'get_rate'), **kwargs)


    def status(self, txn_ref, **kwargs):
        """
        Retrieve the status and various information about a call

        :param txn_ref: The transaction reference for the call
        
        :returns: Return :class:`hoiio.service.Response`
        """     
        print 'Status of %s' % (txn_ref)
        kwargs['txn_ref'] = txn_ref
        return self.make_request(api_endpoint('voice', 'query_status'), **kwargs)




from hoiio.service import Service
from hoiio.service import api_endpoint

class Number(Service):
    """ 
    Provide Number services such as subscribing and configuring the numbers.
    """
    
    def __init__(self):
        pass

    def available_countries(self, **kwargs):
        """
        Returns the countries that have Hoiio numbers.

        :returns: Return :class:`hoiio.service.Response`
        """
        res = self.make_request(api_endpoint('number', 'get_countries'), **kwargs)
        # We make sure states attribute is None if not present 
        for country in res.entries:
            if not hasattr(country, 'states'):
                country.states = None
        return res


    def available_numbers(self, country, state=None, **kwargs):
        """
        Returns the available numbers for purchase
        
        :param country: The country with available numbers
        :param state: The state in the country. Required for countries with states.

        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['country'] = country
        if state != None:
            kwargs['state'] = state
        return self.make_request(api_endpoint('number', 'get_choices'), **kwargs)


    def rate(self, country, **kwargs):
        """
        Returns the subscription cost of the country

        :param country: Subscription cost of the country

        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['country'] = country
        return self.make_request(api_endpoint('number', 'get_rates'), **kwargs)


    def subscribe(self, number, duration, **kwargs):
        """
        Subscribe a number

        :param number: The number to subscribe to
        :param duration: The number of months to subscribe to
        :type duration: 1, 3, 12 or auto_extend

        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['number'] = number
        kwargs['duration'] = duration
        return self.make_request(api_endpoint('number', 'subscribe'), **kwargs)

    # Not useful since API will override
    # def configure_voice(self, number, notify_url, **kwargs):
    #   """
    #   Configure the Notify URL of a number for voice call. The Notify URL is needed for Hoiio to post notification to your server when a call is received at the number.
    #   """
    #   return self.configure(number, mode='voice', forward_to=notify_url)

    # def configure_fax(self, number, notify_url, **kwargs):
    #   """
    #   Configure the Notify URL of a number for fax. The Notify URL is needed for Hoiio to post notification to your server when a fax is received at the number.
    #   """
    #   return self.configure(number, mode='fax', forward_to=notify_url)

    # def configure_sms(self, number, notify_url, **kwargs):
    #   """
    #   Configure the Notify URL of a number for SMS. The Notify URL is needed for Hoiio to post notification to your server when an SMS is received at the number.
    #   """
    #   return self.configure(number, forward_sms_to=notify_url)

    def configure(self, number, **kwargs):
        """
        Configure a number. It is recommended to use configure_voice, configure_fax or configure_sms instead of this.

        :param number: The number to configure
        :param forward_to: The notify_url when there is an incoming call (could be voice or fax depending on mode).
        :param forward_sms_to: The notify_url when there is an incoming SMS
        :param mode: The mode for incoming call. It can either be voice or fax (but not both). Default to voice.
        :type mode: voice or fax

        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['number'] = number
        return self.make_request(api_endpoint('number', 'update_forwarding'), **kwargs)


    def subscribed_numbers(self, **kwargs):
        """
        Retrieve the list of subscribed numbers

        :returns: Return :class:`hoiio.service.Response`
        """
        return self.make_request(api_endpoint('number', 'get_active'), **kwargs)






import urllib
import urllib2

import service.voice
import service.sms
import service.fax
import service.ivr
import service.number
import service.account

class Hoiio(object):

    # Services as class attributes
    voice = service.voice.Voice()
    sms = service.sms.Sms()
    number = service.number.Number()
    fax = service.fax.Fax()
    ivr = service.ivr.Ivr()
    account = service.account.Account()

    services = [voice, sms, number, fax, ivr, account]

    @staticmethod
    def init(app_id, access_token):
        for service in Hoiio.services:
            service.set_auth(app_id, access_token)
            service._Hoiio = Hoiio

    # Phone number prefix
    _prefix = '1'

    # http://stackoverflow.com/questions/128573/using-property-on-classmethods
    class __metaclass__(type):
        @property
        def prefix(cls):
                return cls._prefix
        @prefix.setter
        def prefix(cls, value):
                if value.startswith('+'):
                    value = value[1:]
                cls._prefix = value

class CallStatus:
    """ The Call Status"""
    ANSWERED = 'answered'
    UNANSWERED = 'unanswered'
    FAILED = 'failed'
    BUSY = 'busy'
    ONGOING = 'ongoing'

class SmsStatus:
    """ The SMS Status"""
    QUEUED = 'queued'
    DELIVERED = 'delivered'
    FAILED = 'failed'
    ERROR = 'error'
    RECEIVED = 'received'

class FaxStatus:
    """ The Fax Status"""
    ANSWERED = 'answered'
    UNANSWERED = 'unanswered'
    FAILED = 'failed'
    BUSY = 'busy'
    ONGOING = 'ongoing'





from hoiio.service import Service
from hoiio.service import api_endpoint

class Ivr(Service):
    """ 
    Provide IVR services such as Dial, Gather, Transfer, Hangup, etc.
    """
    
    def __init__(self):
        pass

    def dial(self, dest, **kwargs):
        """
        Make a call out to a phone number

        :param dest: The phone number to call to
        
        :param msg: The message to play when the phone is answered
        :param caller_id: The caller ID when connected to the phone number
        :param max_duration: The maximum duration for the call, afterwhich it will hangup automatically
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['dest'] = dest
        return self.make_request(api_endpoint('ivr/start', 'dial'), **kwargs)


    def play(self, session, msg, **kwargs):
        """
        Play a message over the phone call

        :param session: The IVR session
        :param msg: The message to play

        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['session'] = session
        kwargs['msg'] = msg
        return self.make_request(api_endpoint('ivr/middle', 'play'), **kwargs)
                

    def gather(self, session, **kwargs):
        """
        Gather a keypad response from the user on the phone

        :param session: The IVR session
        
        :param msg: The message to play as you gather 
        :param max_digits: The maximum number of digits to gather from
        :param timeout: The call will hangup if no response after timeout (in seconds)
        :param attempts: The number of times the message will be repeated
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['session'] = session
        return self.make_request(api_endpoint('ivr/middle', 'gather'), **kwargs)
        

    def record(self, session, **kwargs):
        """
        Record a voice message

        :param session: The IVR session
        
        :param msg: The message to play before you record
        :param max_duration: The maximum duration (in seconds) for the recorded voice message
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['session'] = session
        return self.make_request(api_endpoint('ivr/middle', 'record'), **kwargs)

    def monitor(self, session, **kwargs):
        """
        Record the whole phone conversation from this point onwards

        :param session: The IVR session
        
        :param msg: The message to play as you record
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['session'] = session
        return self.make_request(api_endpoint('ivr/middle', 'monitor'), **kwargs)


    def transfer(self, session, dest, **kwargs):
        """
        Transfer the call to another phone number or a conference room

        :param session: The IVR session
        :param dest: The phone number or conference room. If you are transferring to a conference room, prefix with a room: eg. room:R1234.
        
        :param msg: The message to play as you record
        :param caller_id: The caller ID when connected to the dest
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['session'] = session
        kwargs['dest'] = dest
        return self.make_request(api_endpoint('ivr/end', 'transfer'), **kwargs)

    def hangup(self, session, **kwargs):
        """
        Hangup a phone call

        :param session: The IVR session
        
        :param msg: The message to play before you hangup
        :param tag: Your own reference tag for this transaction
        :param notify_url: A notification URL for Hoiio to call to your web server
        
        :returns: Return :class:`hoiio.service.Response`
        """
        kwargs['session'] = session
        return self.make_request(api_endpoint('ivr/end', 'hangup'), **kwargs)






__version_info__ = ('0', '1', '3')
__version__ = '.'.join(__version_info__)
__author__ = 'Junda Ong'
__author_email__ = 'junda@hoiio.com'


import os
from setuptools import setup, find_packages


setup(
    name = "hoiio",
    version = __version__,
    author = __author__,
    author_email = __author_email__,
    description = "Python SDK for Hoiio API",
    license = "MIT",
    keywords = "hoiio voice sms conference fax ivr",
    url = "http://developer.hoiio.com",
    install_requires = ["requests"],
    packages = find_packages(),
    classifiers = [
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.5",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Communications :: Telephony",
    ],
)



import requests
import datetime

from hoiio.exceptions import HoiioException

HOIIO_API_ENDPOINT = 'https://secure.hoiio.com/open/'


class Service(object):

    # Refers back to Hoiio
    _Hoiio = None

    def __init__(self, app_id, access_token):
        setAuth(app_id, access_token)

    def set_auth(self, app_id, access_token):   
        self.app_id = app_id
        self.access_token = access_token

    def make_request(self, url, **kwargs):
        self.validate_auth()

        kwargs['app_id'] = self.app_id
        kwargs['access_token']  = self.access_token
        for key in kwargs:
            print '%s: %s' % (key, kwargs[key])

        # We add prefix if missing
        if 'dest' in kwargs:
            kwargs['dest'] = self.e164_format(kwargs['dest'])

        r = requests.get(url, params=kwargs)
        # print 'Response: %s' % r.text
        
        return Response(r)

    def validate_auth(self):
        if not hasattr(self, 'app_id') or not hasattr(self, 'access_token') or (self.app_id == None) or (self.access_token == None):
            raise HoiioException('App ID and Access Token not init')

    def e164_format(self, phone):
        if not phone.startswith('+'):
            phone = '+' + _Hoiio.prefix + phone
        return phone


class Response:
    """
    A class to encapsulate the API response. It also contains the Response class
    from requests.

    response = Response(r)
    response.status
    """

    def __init__(self, response):
        """
        response is from Requests
        """
        self.response = response
        self.json = response.json
        self.text = response.text
        try:
            for key in response.json:
                value = response.json[key]
                setattr(self, key, sanitize(key, value))

        except Exception, e:
            # It is possible that json is empty and throws: TypeError: 'NoneType' object is not iterable
            print 'Exception: %s' % e
            import traceback
            traceback.print_exc()
            raise HoiioException

    def is_success(self):
        if self.json['status'] == 'success_ok':
            return True
        return False





def api_endpoint(*args):
    """ Returns the Hoiio API endpoint URL. The args are joined by '/'. """
    endpoint = HOIIO_API_ENDPOINT + '/'.join(args)
    return endpoint
    

def sanitize(key, value):
    """ Return a value (type str) in the correct type (int, float, datetime or str) """
    
    # We do special handling for response keys
    # list (str was delimited by commas)
    if key == 'txn_refs':
        return value.split(',')
    
    # datetime
    elif key in ('date', 'expiry'):
        return str_to_date(value, key=key)

    # int
    elif key in ('split_count', 'duration', 'talktime', 'entries_count', 'total_entries_count'):
        return int(value)

    # float
    elif key in ('rate', 'total_cost', 'debit', 'balance', 'points', 'bonus'):
        return float(value)

    # bool
    elif key in ('auto_extend_status',):
        if value == 'disabled':
            return True
        else:
            return False

    # list of objects
    elif key == 'entries':
        # Turns each entry (a dict) into an object
        i = 0
        for entry in value:
            obj = obj_dic(entry)
            value[i] = obj
            i += 1
        return value

    else:
        return value


# http://stackoverflow.com/questions/1305532/convert-python-dict-to-object
# Specially for Hoiio: Added sanitize
def obj_dic(d):
    top = type('new', (object,), d)
    seqs = tuple, list, set, frozenset
    for i, j in d.items():
        if isinstance(j, dict):
            setattr(top, i, obj_dic(j))
        elif isinstance(j, seqs):
            setattr(top, i, 
                    type(j)(obj_dic(sj) if isinstance(sj, dict) else sj for sj in j))
        else:
            setattr(top, i, sanitize(i, j))
    return top




# Util methods
# Maybe move to hoiio module, or utils.py
def str_to_date(s, key=None):
    """ Converts a string (Hoiio format) to datetime """
    # Due to a bug from API, date might have micro sec (after .)
    if key == 'expiry':
        f = "%Y-%m-%d"
    elif '.' in s:
        f = "%Y-%m-%d %H:%M:%S.%f"
    else:
        f = "%Y-%m-%d %H:%M:%S"
    return datetime.datetime.strptime(s, f)

def date_to_str(d):
    """ Converts a datetime to string (Hoiio format) """
    # Due to a bug from API, date might have micro sec (after .)
    f = "%Y-%m-%d %H:%M:%S"
    return d.strftime(f)



from hoiio.service import Service
from hoiio.service import api_endpoint


class Account(Service):
    """ 
    Provide account related services such as retrieve the credit balance and account information
    """
    
    def __init__(self):
        pass

    def balance(self, **kwargs):
        """
        Retrieve the credit balance
        
        :returns: Return :class:`hoiio.service.Response`
        """
        return self.make_request(api_endpoint('user', 'get_balance'), **kwargs)
       

    def info(self, **kwargs):
        """
        Retrieve account info
        
        :returns: Return :class:`hoiio.service.Response`
        """
        res = self.make_request(api_endpoint('user', 'get_info'), **kwargs)
        # To be consistent with Hoiio.number.available_countries, we remove + 
        if res.prefix.startswith('+'):
        	res.prefix = res.prefix[1:]
        return res 
       
